@@ -75,6 +75,7 @@ struct Configuration {
 	unsigned disable_cap:1;
 	unsigned check_target_nick_bans:1;
 	HideBanReasonOption hide_ban_reason;
+	int hide_killed_by;
 	char *link_bindip;
 	long throttle_period;
 	char throttle_count;
@@ -83,6 +84,7 @@ struct Configuration {
 	long oper_modes;
 	char *oper_snomask;
 	char *oper_auto_join_chans;
+	char *oper_vhost;
 	char *allow_user_stats;
 	OperStat *allow_user_stats_ext;
 	int ping_warning;
@@ -121,7 +123,7 @@ struct Configuration {
 	char *spamfilter_ban_reason;
 	char *spamfilter_virus_help_channel;
 	char spamfilter_vchan_deny;
-	SpamExcept *spamexcept;
+	SecurityGroup *spamfilter_except;
 	char *spamexcept_line;
 	long spamfilter_detectslow_warn;
 	long spamfilter_detectslow_fatal;
@@ -185,16 +187,29 @@ struct Configuration {
 	int dns_client_retry;
 	int dns_dnsbl_timeout;
 	int dns_dnsbl_retry;
+	int send_isupport_updates;
+	int utf8_only;
 };
 
 extern MODVAR Configuration iConf;
 extern MODVAR Configuration tempiConf;
 extern MODVAR int ipv6_disabled;
 
+/** The best practices block.
+ * Note that the *_hits stuff is dynamically set and used for 'checks'.
+ * The rest is config... oh except for listen_nontls_port.. sigh.
+ */
 typedef struct BestPractices BestPractices;
 struct BestPractices {
-	int hashed_passwords;		/**< Use hashed passwords */
-	int hashed_passwords_hits;	/**< How many times advice has been given (like 'warnings' but it is advice) */
+	int hashed_passwords;			/**< Use hashed passwords */
+	int hashed_passwords_hits;		/**< - How many times advice has been given (like 'warnings' but it is advice) */
+	int trusted_cert;			/**< Uses a SSL cert issued by a CA */
+	int trusted_cert_hits;			/**< - How many times advice has been given (like 'warnings' but it is advice) */
+	int trusted_cert_valid_hostname;	/**< Uses a SSL cert issued by a CA that is valid for me::name */
+	int trusted_cert_valid_hostname_hits;	/**< - How many times advice has been given (like 'warnings' but it is advice) */
+	int listen_tls_only;			/**< listen { } blocks only with tls */
+	int listen_nontls_port;			/**< first non-tls port found */
+	int listen_nontls_port_hits;		/**< - How many times advice has been given (like 'warnings' but it is advice) */
 };
 extern MODVAR BestPractices bestpractices;
 
@@ -275,6 +290,8 @@ extern MODVAR BestPractices bestpractices;
 
 #define UHNAMES_ENABLED	iConf.uhnames
 
+#define UTF8ONLY	iConf.utf8_only
+
 /** Used for testing the set { } block configuration.
  * It tests if a setting is present and is also used for duplicate checking.
  */
@@ -352,4 +369,5 @@ struct SetCheck {
 	unsigned has_min_nick_length:1;
 	unsigned has_nick_length:1;
 	unsigned has_hide_ban_reason:1;
+	unsigned has_hide_killed_by:1;
 };
